package sun.rmi.transport;

import java.security.ProtectionDomain;
import java.security.Permissions;
import java.rmi.server.LogStream;
import java.io.ObjectOutput;
import java.rmi.server.ServerNotActiveException;
import java.rmi.server.RemoteServer;
import sun.rmi.server.UnicastServerRef;
import java.rmi.NoSuchObjectException;
import java.security.PrivilegedActionException;
import java.rmi.Remote;
import sun.rmi.server.Dispatcher;
import java.security.PrivilegedExceptionAction;
import java.io.IOException;
import java.rmi.MarshalException;
import java.rmi.server.RemoteCall;
import java.rmi.RemoteException;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.security.AccessControlContext;
import java.rmi.server.ObjID;
import sun.rmi.runtime.Log;

public abstract class Transport
{
    static final int logLevel;
    static final Log transportLog;
    private static final ThreadLocal<Transport> currentTransport;
    private static final ObjID dgcID;
    private static final AccessControlContext SETCCL_ACC;
    
    public Transport() {
    }
    
    private static String getLogLevel() {
        return (String)AccessController.doPrivileged(new GetPropertyAction("sun.rmi.transport.logLevel"));
    }
    
    public abstract Channel getChannel(Endpoint p0);
    
    public abstract void free(Endpoint p0);
    
    public void exportObject(Target target) throws RemoteException {
        target.setExportedTransport(this);
        ObjectTable.putTarget(target);
    }
    
    protected void targetUnexported() {
    }
    
    static Transport currentTransport() {
        return (Transport)Transport.currentTransport.get();
    }
    
    protected abstract void checkAcceptPermission(AccessControlContext p0);
    
    private static void setContextClassLoader(ClassLoader classLoader) {
        AccessController.doPrivileged(()/*  */ -> {
            Thread.currentThread().setContextClassLoader(p0);
            return null;
        }, Transport.SETCCL_ACC);
    }
    
    public boolean serviceCall(final RemoteCall remoteCall) {
        try {
            ObjID read;
            try {
                read = ObjID.read(remoteCall.getInputStream());
            }
            catch (IOException ex) {
                throw new MarshalException("unable to read objID", ex);
            }
            Target target = ObjectTable.getTarget(new ObjectEndpoint(read, read.equals(Transport.dgcID) ? null : this));
            final Remote impl;
            if (target == null || (impl = target.getImpl()) == null) {
                throw new NoSuchObjectException("no such object in table");
            }
            final Dispatcher dispatcher = target.getDispatcher();
            target.incrementCallCount();
            try {
                Transport.transportLog.log(Log.VERBOSE, "call dispatcher");
                final AccessControlContext accessControlContext = target.getAccessControlContext();
                ClassLoader contextClassLoader = target.getContextClassLoader();
                ClassLoader contextClassLoader2 = Thread.currentThread().getContextClassLoader();
                try {
                    setContextClassLoader(contextClassLoader);
                    Transport.currentTransport.set(this);
                    try {
                        AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                            @Override
                            public Void run() throws IOException {
                                Transport.this.checkAcceptPermission(accessControlContext);
                                dispatcher.dispatch(impl, remoteCall);
                                return null;
                            }
                        }, accessControlContext);
                    }
                    catch (PrivilegedActionException ex2) {
                        throw (IOException)ex2.getException();
                    }
                }
                finally {
                    setContextClassLoader(contextClassLoader2);
                    Transport.currentTransport.set(null);
                }
            }
            catch (IOException ex3) {
                Transport.transportLog.log(Log.BRIEF, "exception thrown by dispatcher: ", ex3);
                return false;
            }
            finally {
                target.decrementCallCount();
            }
        }
        catch (RemoteException t) {
            if (UnicastServerRef.callLog.isLoggable(Log.BRIEF)) {
                String string = "";
                try {
                    string = "[" + RemoteServer.getClientHost() + "] ";
                }
                catch (ServerNotActiveException ex4) {}
                UnicastServerRef.callLog.log(Log.BRIEF, string + "exception: ", t);
            }
            try {
                ObjectOutput resultStream = remoteCall.getResultStream(false);
                UnicastServerRef.clearStackTraces(t);
                resultStream.writeObject(t);
                remoteCall.releaseOutputStream();
            }
            catch (IOException ex5) {
                Transport.transportLog.log(Log.BRIEF, "exception thrown marshalling exception: ", ex5);
                return false;
            }
        }
        return true;
    }
    
    static {
        logLevel = LogStream.parseLevel(getLogLevel());
        transportLog = Log.getLog("sun.rmi.transport.misc", "transport", Transport.logLevel);
        currentTransport = new ThreadLocal();
        dgcID = new ObjID(2);
        Permissions permissions = new Permissions();
        permissions.add(new RuntimePermission("setContextClassLoader"));
        SETCCL_ACC = new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, permissions) });
    }
}
